/* 
 * Copyright 1991 Oliver Jones
 * Permission to use, copy, modify, distribute, and sell this software
 * for any purpose is hereby granted without fee.  No representations
 * are made by anyone about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty. 
 */

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include "image.h"
#include "object.h"

/* make it easier to handle GC flags */
#define GC_ALL_FLAGS  ((1L<<(GCLastBit+1L))-1L)
	
#define EQ(a,b) (0 == strcmp((a),(b)))

struct DrawContext {
  Widget          draw;  /* draw widget */
  Widget          top;   /* top of tree */
  char *          tool;  /* name of tool */
  RenderContext * rend;  /* rendering context */
  DrawObject *    curob; /* currently active object */
  DrawObject *    oblist;/* list of objects */
};

/* private data declarations */
static DrawContext * current = NULL;

/* context-manipulation routines */
/* create a draw context using the specified widget */
DrawContext * NewDrawContext (w,top)
     Widget w;
     Widget top;
{
  DrawContext * c = 
    (DrawContext *) XtCalloc (sizeof(DrawContext),1);
  if (!c) abort ();
  c->draw = w;
  c->top  = top;
  c->tool = "";
  c->curob = NULL;
  c->oblist = NULL;
  c->rend = NewRenderContext (w);
  if (!c->rend) abort ();
  return c;
}

/* set the current draw context for a display */
/* ARGSUSED */
DrawContext * SetDrawContext (w, set)
     Widget        w;
     DrawContext * set;
{
  DrawContext * c = current;
  current = set;
  return c;
}

/* get the current draw context for a display */
/* ARGSUSED */
DrawContext * GetDrawContext (w)
     Widget w;
{
  if (!current) abort();
  return current;
}


/*ARGSUSED */
void File (w, ev, params, num_params)
     Widget w;
     XEvent * ev;
     String params[];
     Cardinal * num_params;
{
  DrawContext * c = GetDrawContext(w);
  Window    win = XtWindow  (c->draw);
  Display * dpy = XtDisplay (c->draw);

  printf ("File %s\n", params[0]);
}

/* ARGSUSED */
void Edit (w, ev, params, num_params)
     Widget w;
     XEvent * ev;
     String params[];
     Cardinal * num_params;
{
  register DrawContext * c = GetDrawContext(w);
  register RenderContext *r = c->rend;
  char * order  = params[0];
  Arg arg[3];
  Dimension width, height;
  XRectangle rect;
  register DrawObject * o, *nexto;

  /* look up widget size */
  XtSetArg (arg[0], XtNwidth,  &width);
  XtSetArg (arg[1], XtNheight, &height);
  XtGetValues (c->draw, arg, 2);
  rect.x = 0;    rect.y = 0;
  rect.width = width;
  rect.height = height;

  if (EQ("clear", order)) {

    for (o = c->oblist;	 o; o = nexto ) {
      nexto = NextObject(o);
      FreeObject (o);
    }
    c->oblist = NULL;

    if (c->curob) 
      FreeObject (c->curob);
    c->curob = NULL;
  
    RenderClear (r, &rect, False);
    
  }
  else {
    printf ("Edit %s\n", params[0]);
  }
}

/* ARGSUSED */
void Choose (w, ev, params, num_params)
     Widget w;
     XEvent * ev;
     String params[];
     Cardinal * num_params;
{
  register DrawContext * c = GetDrawContext(w);
  c->tool = params[0];
}

/* ARGSUSED */
void Redraw (w, ev, params, num_params)
     Widget w;
     XEvent * ev;
     String params[];
     Cardinal * num_params;
{
  register DrawContext * c = GetDrawContext(w);
  register DrawObject * o;

  for (o = c->oblist;
       o;
       o = NextObject(o)) 
    PaintObject (o,Unchanged);

  if (c->curob) 
    PaintObject (c->curob, Unchanged);
  
}

/* ARGSUSED */
void Start (w, ev, params, num_params)
     Widget w;
     XEvent * ev;
     String params[];
     Cardinal * num_params;
{
  register DrawContext * c = GetDrawContext(w);
  register RenderContext * r = c->rend;
  Vector v;

  EventVector (ev, &v);
  if (!c->curob) c->curob = NewObject (r, c->tool, &v, &v);
  PaintObject (c->curob, Echoed);
}

/* ARGSUSED */
void Move (w, ev, params, num_params)
     Widget w;
     XEvent * ev;
     String params[];
     Cardinal * num_params;
{
  register DrawContext * c = GetDrawContext(w);
  Vector v;
  EventVector (ev, &v);

  if (c->curob) {
    ResizeObject (c->curob, &v);
  }
}

/* ARGSUSED */
void End (w, ev, params, num_params)
     Widget w;
     XEvent * ev;
     String params[];
     Cardinal * num_params;
{
  register DrawContext * c = GetDrawContext(w);
  Vector v;

  EventVector (ev, &v);

  if (c->curob) {

    /* undraw object */
    PaintObject (c->curob, Erased);

    /* final size */
    ResizeObject (c->curob, &v);

    /* draw permanently */
    PaintObject (c->curob, Drawn);

    /* put in permanent list */
    LinkObject (c->curob, c->oblist);
    c->oblist = c->curob;
    c->curob = NULL;
  }
}

/* ARGSUSED */
void Debug (w, ev, params, num_params)
     Widget w;
     XEvent * ev;
     String params[];
     Cardinal * num_params;
{

}



