/* 
 * Copyright 1991 Oliver Jones
 * Permission to use, copy, modify, distribute, and sell this software
 * for any purpose is hereby granted without fee.  No representations
 * are made by anyone about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty. 
 */

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include "render.h"
#include "image.h"
#include "object.h"

struct RenderContext {
  Display *     dpy;      /* display pointer for this object */
  Window        win;      /* window for this object */
  Dimension     depth;    /* window's depth */
  XGCValues     gcv;      /* default GC values */
  XFontStruct * font;     /* default font */
  GC            drawgc;   /* GC for drawing */
  GC            echogc;   /* GC for echoing */
  GC            auxgc;    /* extra GC */
};

/* defaults from command line */
static XtResource resources[] = {
  {XtNfont, XtCFont, XtRFontStruct, sizeof (XFontStruct *),
     XtOffset(RenderContext *, font), 
     XtRString, "9x15" },
  {XtNforeground, XtCForeground, XtRPixel, sizeof (int),
     XtOffset(RenderContext *, gcv.foreground), XtRString, "Black"}, 
  {XtNbackground, XtCBackground, XtRPixel, sizeof (int),
     XtOffset(RenderContext *, gcv.background), XtRString, "White"}
};

/* create a draw context using the specified widget */
RenderContext * NewRenderContext (w)
     Widget w;
{
  Window    win = XtWindow  (w);
  Display * dpy = XtDisplay (w);
  RenderContext * r = 
    (RenderContext *) XtCalloc (sizeof(RenderContext),1);

  XtGetApplicationResources (w, r, 
			     resources, XtNumber (resources),
			     NULL, 0);

  r->gcv.font = r->font->fid;
  r->drawgc = XCreateGC (dpy, win, 
		       GCFont | GCForeground | GCBackground,
		       &r->gcv);
  r->echogc = XCreateGC (dpy, win, 0, NULL);
  XSetForeground (dpy, r->echogc,
		  r->gcv.foreground ^ r->gcv.background);
  XSetFunction (dpy, r->echogc, GXxor);
  r->auxgc = XCreateGC (dpy, win, 0, NULL);
  r->dpy = dpy;
  r->win = win;
  return r;
}

void RenderClear (r, rect, repaint)
     RenderContext * r;
     XRectangle *rect;
     Boolean repaint;
{
  XClearArea (r->dpy, r->win, 
	      rect->x, rect->y, 
	      rect->width, rect->height, 
	      repaint);
}

GC RenderDraw (r)
     RenderContext * r;
{
  return r->drawgc;
}

GC RenderEcho (r)
     RenderContext * r;
{
  return r->echogc;
}

GC RenderAux (r)
     RenderContext * r;
{
  return r->auxgc;
}

Display * RenderDisplay (r)
     RenderContext * r;
{
  return r->dpy;
}

Window RenderWindow (r)
     RenderContext * r;
{
  return r->win;
}

void RenderClip (r, region)
     RenderContext * r;
     Region region;
{
  if (region) 
    XSetRegion (r->dpy, r->drawgc, region);
  else
    XSetClipMask (r->dpy, r->drawgc, None);

  XCopyGC (r->dpy, r->drawgc, 
	   GCClipMask | GCClipXOrigin | GCClipYOrigin,
	   r->echogc);
  XCopyGC (r->dpy, r->drawgc, 
	   GCClipMask | GCClipXOrigin | GCClipYOrigin,
	   r->auxgc);
}

