/*
 * Copyright 1991 Oliver Jones
 * Portions Copyright 1989 Massachusetts Institute of Technology
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the name of M.I.T. not
 * be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  No
 * representation are made by anyone about the suitability of this
 * software for any purpose.  It is provided "as is" without express
 * or implied warranty.
 * Adapted from "bitmap" code by Davor Matic and Chris Peterson
 */

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/SmeBSB.h>
#include "image.h"

static char usage[] = { "Usage: sample [Xt options] \n" };

void Quit   ();

static XtActionsRec actions [] = {

  {"File",     (XtActionProc) File}, 
  {"Quit",     (XtActionProc) Quit},
  {"Edit",     (XtActionProc) Edit},
  {"Choose",   (XtActionProc) Choose},

  {"Redraw", (XtActionProc) Redraw},
  {"Start",  (XtActionProc) Start},
  {"Move",   (XtActionProc) Move},
  {"End",    (XtActionProc) End},

  {"Debug",  (XtActionProc) Debug},
};


typedef struct ButtonDesc {
  XtActionProc act;
  String       name;
  Widget       w
} ButtonDesc;

static ButtonDesc fileMenu[] = {
  { (XtActionProc)File, "new" },
  { (XtActionProc)File, "open"},
  { (XtActionProc)File, "close"},
  { (XtActionProc)NULL,  NULL},
  { (XtActionProc)File, "save"},
  { (XtActionProc)File, "saveas"},
  { (XtActionProc)NULL,  NULL},
  { (XtActionProc)Quit, "quit"},
};

static ButtonDesc editMenu[]  = {
  { (XtActionProc)Edit,  "undo"},
  { (XtActionProc)NULL,   NULL},
  { (XtActionProc)Edit,  "cut"},
  { (XtActionProc)Edit,  "copy"},
  { (XtActionProc)Edit,  "paste"},
  { (XtActionProc)NULL,   NULL},
  { (XtActionProc)Edit,  "clear"},
};

static ButtonDesc paletteMenu[]  = {
  { (XtActionProc)Choose, "drawrect"},
  { (XtActionProc)Choose, "fillrect"},
  { (XtActionProc)Choose, "line"},
  { (XtActionProc)Choose, "arrow"},
};

String resources[] = { 
  NULL,
};

/*ARGSUSED*/
void Quit (w, ev, params, num_params)
     Widget w;
     XEvent * ev;
     String params[];
     Cardinal * num_params;
{
  if (w)
    XtDestroyApplicationContext(XtWidgetToApplicationContext(w));
  exit(0);
}

/* ARGSUSED */
void
MenuCallback (w, button, state)
     Widget       w;
     ButtonDesc * button;
     int          state;
{
  if (XtClass(w) == toggleWidgetClass && state==0) {
    /*EMPTY*/ /* ignore toggle resets */
  }
  else {
    if (button && button->act) 
      (*button->act) (w, NULL, &button->name, 1);
  }
}


static void
MakeMenu (menubar, menuName, buttonName, menu, num)
     Widget     menubar;
     String     menuName;
     String     buttonName;
     ButtonDesc menu[];
     Cardinal   num;
{
  Widget bw;
  Widget mw;
  int    i;
  
  mw = XtCreatePopupShell(menuName,
			  simpleMenuWidgetClass, 
			  menubar, NULL, 0);
  bw = XtCreateManagedWidget(buttonName,
			     menuButtonWidgetClass, 
			     menubar, NULL, 0);
  for (i = 0; i < num; i++) {
    char * name = menu[i].name;
    if (!name) name = "line";
    bw = XtCreateManagedWidget(name,
			       menu[i].name ?
			         smeBSBObjectClass :
			         smeLineObjectClass,
			       mw, NULL, 0);
    XtAddCallback (bw, XtNcallback, MenuCallback,
		   (XtPointer)&menu[i]);
    menu[i].w = bw;
  }
}

XrmOptionDescRec opt [] = {
  {"-highlight", "*highlight", XrmoptionSepArg, NULL }
};

void main(argc, argv)
     int    argc;
     char  *argv[];
{
  int i;
  XtAppContext app;
  Widget top;
  Widget parent;
  Widget menubar;
  Widget palette;
  Widget pane;
  Widget image;
  Widget radio_group;
  char * radio_data;
  Atom  wm_delete_window;
  
  Widget w;
  
  top = XtAppInitialize(&app, "Sample",
			opt, XtNumber(opt),
			&argc, argv, resources, NULL, 0);
  
  if (argc > 2) {
    fprintf(stderr, usage);
    exit (0);
  }
  
  XtAppAddActions(app, actions, XtNumber(actions));
  
  parent = XtCreateManagedWidget("parent", panedWidgetClass,
				 top, NULL, 0);
  
  menubar = XtCreateManagedWidget("menubar", formWidgetClass,
				  parent, NULL, 0);
  
  MakeMenu (menubar, "fileMenu", "fileButton", 
	    fileMenu, XtNumber(fileMenu));
  MakeMenu (menubar, "editMenu", "editButton",
	    editMenu, XtNumber(editMenu));
  
  pane = XtCreateManagedWidget("pane", panedWidgetClass,
			       parent, NULL, 0);
  
  palette = XtCreateManagedWidget("palette", formWidgetClass, 
				  pane, NULL, 0);
  
  for (i = 0; i < XtNumber(paletteMenu); i++) {
    w = XtCreateManagedWidget(paletteMenu[i].name,
			      toggleWidgetClass,
			      palette, NULL, 0);
    XtAddCallback (w, XtNcallback, MenuCallback,
		   (XtPointer)&paletteMenu[i]);
    paletteMenu[i].w = w;
    if (i==1) {
      radio_group = paletteMenu[i].w;
      radio_data  = paletteMenu[i].name;
    }
  }
  
  image = XtCreateManagedWidget("image", widgetClass,
				pane, NULL, 0);
  
  XtRealizeWidget(top);
  
  wm_delete_window = XInternAtom(XtDisplay(top), "WM_DELETE_WINDOW",
				 False);
  (void) XSetWMProtocols (XtDisplay(top), XtWindow(top),
			  &wm_delete_window, 1);
  XtOverrideTranslations
    (top,
     XtParseTranslationTable("<Message>WM_PROTOCOLS: Quit()"));
  
  SetDrawContext(top, NewDrawContext(image, top));
  
  XawToggleSetCurrent(radio_group, radio_data);
  
  XtAppMainLoop(app);
}
